require 'special-giggle/lexer'
require 'special-giggle/parser'
require 'special-giggle/generator'

module SpecialGiggle
  include Lexer
  include Parser
  include Generator

  class GiggleError < StandardError; end

  def self.to_asm(string)
    tokens = Lexer.tokenize(string)
    ast = Parser.parse(tokens)
    code = Generator.generate(ast)
    code
  end

  def self.compile(infile, outfile, tmpdir = "/tmp")
    raise GiggleError, "Input file #{infile} does not found." unless File.exist?(infile)
    file = File.basename(infile, ".*")
    sfile, ofile = File.expand_path("#{file}.s", tmpdir), File.expand_path("#{file}.o", tmpdir)
    File.write(sfile, to_asm(File.read(infile)))
    system("as --64 -o #{ofile} #{sfile}") || (raise GiggleError, "Can't compile with as")
    system("gcc -no-pie -o #{outfile} #{ofile}") || (raise GiggleError, "Can't link with gcc")
  end
end
