module Lexer
  class LexerError < StandardError; end

  class Token
    attr_reader :string, :type

    def initialize(string, type)
      @string, @type = string, type
    end
  end

  Patterns = {
    :Blank          => '[ \t\r]+',
    :NewLine        => '\n+',
    :Print          => 'print',
    :Variable       => '[A-Za-z_][A-Za-z0-9_]*',
    :Number         => '-?[0-9]+',
    :Plus           => '\+',
    :Minus          => '\-',
    :Asterisk       => '\*',
    :Slash          => '\/',
    :Percent        => '\%',
    :Bar            => '\|',
    :Ampersand      => '\&',
    :Caret          => '\^',
    :Tilde          => '\~', 
    :LeftShift      => '<<',
    :RightShift     => '>>',
    :Equal          => '\=',
    :LeftBracket    => '\(',
    :RightBracket   => '\)',
    :SemiColon      => '\;',
  }

  def self.next_token(string, index)
    Patterns.each do |type, pattern|
      res = string.match(pattern, index)
      next if res.nil?
      return [res[0], type, res.end(0)] if res.begin(0) == index
    end
    [nil, nil, nil]
  end

  def self.tokenize(string)
    tokens = Array.new
    index, line, column = 0, 1, 1
    while index < string.length
      rstring, rtype, index = next_token(string, index)
      case rtype
      when nil
        raise LexerError, "Unrecognized character in line #{line}, column #{column}"
      when :Blank
        next
      when :NewLine
        line, column = line + 1, 1
      else
        tokens << Token.new(rstring, rtype)
      end
    end
    tokens
  end
end
